import os, shutil, tempfile

verbose=1
def safesystem(cmd, *args):
	if not hasattr(cmd,"lower"):#not a string
		args = tuple(cmd[1:]) + args
		cmd = cmd[0]
	if verbose:
		print 'test: spawnlp(%s, %s, %s, %s)'%(os.P_WAIT, cmd, cmd, ', '.join(args))
	s = os.spawnlp(os.P_WAIT, cmd, cmd, *args)
	if s: raise Exception, "spawnlp(%s, %s, %s, %s) returned %s"%(os.P_WAIT, cmd, cmd, ', '.join(args), s)

def getoutput(cmd, *args):
	if verbose:
		print 'test: popen%s'%((cmd,)+args,)
	f = os.popen(' '.join((cmd,)+args))
	o = f.read()
	s = f.close()
	if s: raise Exception, "popen%s.close() returned %s(%r)"%((cmd,)+args,s,o)
	return o

def get_recursive_path_list(path, ignore=['CVS','.svn']):
	l = []
	for f in os.listdir(path or os.curdir):
		if f in ignore:
			continue
		f = os.path.join(path, f)
		l.append(f)
		if os.path.isdir(f):
			l.extend(get_recursive_path_list(f, ignore))
	return l

def unpack(filename):
	l1 = os.listdir(os.curdir)
	safesystem('aunpack',filename)
	l2 = os.listdir(os.curdir)
	for d in l2:
		if d not in l1:
			return d
	raise Exception, 'nothing unpacked?'

def listcmp(a, b):
	extra = [f for f in a if f not in b]
	missing = [f for f in b if f not in a]
	if extra or missing:
		return 'extra: %r  missing: %r'%(extra,missing)


class Tarballer:
	def __init__(self, name, format='tar.gz'):
		self.name = name
		self.format = format
		self.flist = {}
		self.vflist = {}
		self.tmpdir = tempfile.mktemp()
		self.tardir = os.path.join(self.tmpdir, 'tars')
		os.makedirs(self.tardir, 0700)
	def __del__(self):
		if hasattr(self,'tmpdir') and os.path.exists(self.tmpdir):
			print 'Tarballer: removing',self.tmpdir
			shutil.rmtree(self.tmpdir)
	
	def file_contents(self,path, ver, extra):
		return 'file %s from ver %s\n%s'%(path,ver,extra or "")
	def tarpath(self, ver):
		return os.path.join(self.tardir, self.name + '-' + ver + '.' + self.format)

	def startver(self, version):
		self.curver = version
	def finishver(self):
		self.vflist[self.curver] = {}
		srcdirname = self.name + '-' + self.curver
		srcdir = os.path.join(self.tmpdir, srcdirname)
		os.mkdir(srcdir, 0700)
		for path,extra in self.flist.items():
			parts = path.split('/')
			if parts[:-1]:#if we are a subdir or a file in a subdir
				fulldir = os.path.join(srcdir, *parts[:-1])
				self.vflist[self.curver][os.path.join(*parts[:-1])]=None
				if not os.path.exists(fulldir):
					os.makedirs(fulldir)
			if parts[-1]:#if we are a file
				fullpath = os.path.join(srcdir, *parts)
				self.vflist[self.curver][os.path.join(*parts)]=extra
				f=open(fullpath,'w')
				f.write(self.file_contents(path,self.curver,extra))
				f.close()
		
		olddir = os.getcwd()
		os.chdir(self.tmpdir)
		try:
			safesystem('apack',self.tarpath(self.curver),srcdirname)
		finally:
			os.chdir(olddir)
		shutil.rmtree(srcdir)
		
	def add(self, path, extrainfo=None):
		self.flist[path]=extrainfo
	def rm(self, path):
		del self.flist[path]
	
	def addver(self, version, add=(), rm=(), addextra={}):
		self.startver(version)
		for p in add:
			self.add(p)
		for p,extra in addextra.items():
			self.add(p,extra)
		for p in rm:
			self.rm(p)
		self.finishver()

	def verify(self, ver, path, ignore=['.cvsignore']):
		olddir = os.getcwd()
		os.chdir(path)
		try:
			allfiles = get_recursive_path_list('')
			extra = [f for f in allfiles if f not in self.vflist[ver].keys() and os.path.split(f)[1] not in ignore]
			missing = [f for f in self.vflist[ver].keys() if f not in allfiles]
			if extra or missing:
				return 'extra files: %r  missing files: %r'%(extra,missing)
			for f,extra in self.vflist[ver].items():
				assert os.path.exists(f)
				if os.path.isfile(f):
					if open(f).read() != self.file_contents(f,ver,extra):
						return '%s contents (%r) != expected (%r)'%(f,open(f).read(),self.file_contents(f,ver,extra))
					
		finally:
			os.chdir(olddir)

